<?php class welldrop extends baseobject { 
	
	protected static $fields=array(
			'name'=>validator::REQUIRED,
			'platewellid'=>array(validator::REQUIRED, validator::INTEGER),
			'constructid'=>validator::INTEGER,
			'dropnumber'=>array(validator::REQUIRED, validator::INTEGER),
			'proteinconcentrationamount'=>validator::FLOAT,
			'proteinconcentrationunit'=>validator::ANY,
			'proteinbuffer'=>validator::ANY,
			'proteinsolutionamount'=>validator::FLOAT,
			'proteinsolutionunit'=>validator::ANY,
			'wellsolutionamount'=>validator::FLOAT,
			'wellsolutionunit'=>validator::ANY,
	);
	
	protected static $helpTexts=array(
			'name'=>'A unique name for this drop',
			'platewellid'=>'The parent well',
			'constructid'=>'The protein construct in this drop',
			'dropnumber'=>'Number of the drop within the well'
	);
		
	protected static $adminSelect='SELECT SQL_CALC_FOUND_ROWS welldrop.*, construct.name AS constructname, protein.id AS proteinid, protein.name AS proteinname
			FROM welldrop  LEFT JOIN construct ON construct.id=welldrop.constructid LEFT JOIN protein ON protein.id=construct.proteinid
			WHERE 1=1';
	protected static $normalSelect='SELECT SQL_CALC_FOUND_ROWS welldrop.*, construct.name AS constructname, protein.id AS proteinid, protein.name AS proteinname
			FROM welldrop LEFT JOIN construct ON construct.id=welldrop.constructid LEFT JOIN protein ON protein.id=construct.proteinid 
			LEFT JOIN project ON project.id=welldrop.projectid
			WHERE 1=1';

    /**
     * @param int $id The ID of the well drop.
     * @param array $request The request parameters, e.g., pagination.
     * @return array
     * @noinspection PhpUnusedParameterInspection
     */
	public static function gettimecourseimages($id,$request=array()){
        return database::queryGetAll(dropimage::getSelectClause().' AND welldropid=:wd
                ORDER BY imagingsession.imageddatetime ASC',
                array(':wd'=>$id));
	}

    /**
     * @param int $id The ID of the well drop.
     * @param array $request The request parameters, e.g., pagination.
     * @return array
     * @throws BadRequestException
     * @throws NotFoundException
     * @throws ServerException
     */
	public static function getcrystals($id, $request=array()){
		return crystal::getByProperty('welldropid', $id, $request);
	}

    public static function getByPlateIdAndRowColumnDrop($plateId, $rowNumber, $colNumber, $dropNumber){
        return database::queryGetOne('
	        SELECT welldrop.*
	        FROM welldrop, platewell 
            WHERE platewell.id=welldrop.platewellid
            AND platewell.row=:row AND platewell.col=:col
            AND platewell.plateid=:plateid AND welldrop.dropnumber=:drop
	    ',array(
            ':plateid'=>$plateId,
            ':row'=>$rowNumber,
            ':col'=>$colNumber,
            ':drop'=>$dropNumber
        ));
    }

    /**
     * @param $id
     * @return void
     * @throws BadRequestException
     * @throws ForbiddenException
     * @throws NotFoundException
     * @throws ServerException
     */
    public static function carryForwardScores($id){
        $wellDrop=welldrop::getById($id);
        if(!$wellDrop){
            throw new NotFoundException('No well drop with ID '.$id);
        }
        $images=welldrop::gettimecourseimages($id);
        if(empty($images) || !isset($images['rows'])){
            return;
        }
        $currentScore=-1;
        foreach($images["rows"] as $image){
            if(!empty($image['latestcrystalscoreid'])){
                $currentScore=$image['latestcrystalscoreid'];
            } else if(-1!==$currentScore){
                dropimage::update($image['id'], array('latestcrystalscoreid'=>$currentScore));
            }
        }
    }

}